import os
import base64
import asyncio
from typing import Union
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.backends import default_backend
from cryptography.fernet import Fernet


class CryptographyManager:
    def __init__(
        self,
        password: Union[str, bytes],
        salt: Union[str, bytes] = None,
        iterations: int = 400000,
    ):
        """
        Initialize the CryptographyManager with the given parameters.

        :param password: The password used for key derivation.
        :param salt: The salt to use for PBKDF2. If None, a new salt is generated.
        :param iterations: The number of iterations for PBKDF2.
        """
        self.password = (
            password if isinstance(password, bytes) else str.encode(password)
        )
        if salt:
            self.salt: bytes = (
                salt
                if isinstance(salt, bytes)
                else str.encode(salt).decode("unicode_escape").encode()
            )
        else:
            self.salt = os.urandom(16)

        self.iterations = iterations
        self.key_length = 32  # Length of the derived key in bytes (for Fernet)

        # Create PBKDF2HMAC instance
        self.kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=self.key_length,
            salt=self.salt,
            iterations=self.iterations,
            backend=default_backend(),
        )

        # Derive a key from the password
        self.key = self.kdf.derive(self.password)

        # Encode the key in Base64 for Fernet
        self.encoded_key = base64.urlsafe_b64encode(self.key)

        # Create Fernet instance with the derived key
        self.fernet = Fernet(self.encoded_key)

    async def __aencrypt(self, message: Union[str, bytes]) -> bytes:
        """
        Encrypt the given message using Fernet asynchronously.

        :param message: The message to encrypt.
        :return: The encrypted message.
        """
        message = message if isinstance(message, bytes) else str.encode(message)
        return await asyncio.to_thread(self.fernet.encrypt, message)

    async def __adecrypt(self, token: bytes) -> bytes:
        """
        Decrypt the given token using Fernet asynchronously.

        :param token: The encrypted message to decrypt.
        :return: The decrypted message.
        """
        message = await asyncio.to_thread(self.fernet.decrypt, token)
        return message.decode()

    def encrypt(self, message: Union[str, bytes]) -> bytes:
        try:
            loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
        except RuntimeError:
            loop = None

        if loop and loop.is_running():
            return self.__aencrypt(message=message)
        else:
            return asyncio.run(self.__aencrypt(message=message))

    def decrypt(self, token: bytes) -> bytes:
        try:
            loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
        except RuntimeError:
            loop = None

        if loop and loop.is_running():
            return self.__adecrypt(token=token)
        else:
            return asyncio.run(self.__adecrypt(token=token))



import os
import pytest
from cryptography.fernet import InvalidToken
from risclog.cryptography import CryptographyManager


@pytest.mark.asyncio
async def test_async_cryptography_manager():
    password = b"fSaCbrwrNZt0TYumhzbAOCQl2Trp2siFqXa-rowb4l8="
    message = b"Secret message!"
    salt = os.urandom(16)

    crypto_manager = CryptographyManager(password=password, salt=salt)

    token = await crypto_manager.encrypt(message)
    assert token is not None

    decrypted_message = await crypto_manager.decrypt(token)
    assert decrypted_message == message.decode()


@pytest.mark.asyncio
async def test_async_cryptography_manager_invalid_token():
    salt = os.urandom(16)
    password = b"fSaCbrwrNZt0TYumhzbAOCQl2Trp2siFqXa-rowb4l8="
    invalid_token = b"InvalidToken"
    crypto_manager = CryptographyManager(password=password, salt=salt)

    with pytest.raises(InvalidToken):
        await crypto_manager.decrypt(invalid_token)


@pytest.mark.asyncio
async def test_async_cryptography_manager_with_salt_not_set():
    password = b"fSaCbrwrNZt0TYumhzbAOCQl2Trp2siFqXa-rowb4l8="
    message = b"Secret message!"

    crypto_manager = CryptographyManager(password=password)

    token = await crypto_manager.encrypt(message)
    assert token is not None

    decrypted_message = await crypto_manager.decrypt(token)
    assert decrypted_message == message.decode()


@pytest.mark.asyncio
async def test_async_cryptography_manager_with_salt_not_set_and_second_cryptography_manager_instance():
    password = b"fSaCbrwrNZt0TYumhzbAOCQl2Trp2siFqXa-rowb4l8="
    message = b"Secret message!"

    crypto_manager = CryptographyManager(password=password)
    token = await crypto_manager.encrypt(message)
    assert token is not None

    crypto_manager = CryptographyManager(password=password)
    with pytest.raises(InvalidToken):
        await crypto_manager.decrypt(token)


def test_cryptography_manager():
    password = b"fSaCbrwrNZt0TYumhzbAOCQl2Trp2siFqXa-rowb4l8="
    message = b"Secret message!"
    salt = os.urandom(16)

    crypto_manager = CryptographyManager(password=password, salt=salt)

    token = crypto_manager.encrypt(message)
    assert token is not None

    decrypted_message = crypto_manager.decrypt(token)
    assert decrypted_message == message.decode()


def test_cryptography_manager_invalid_token():
    salt = os.urandom(16)
    password = b"fSaCbrwrNZt0TYumhzbAOCQl2Trp2siFqXa-rowb4l8="
    invalid_token = b"InvalidToken"
    crypto_manager = CryptographyManager(password=password, salt=salt)

    with pytest.raises(InvalidToken):
        crypto_manager.decrypt(invalid_token)



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("CHANGES.rst") as history_file:
    history = history_file.read()

setup(
    author="riscLOG Solution GmbH",
    author_email="info@risclog.de",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: German",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    description="The CryptographyManager is a tool designed to securely encrypt and decrypt messages and data. It uses modern encryption techniques to protect confidential information.",
    install_requires=[
        # Add your dependencies here
        "risclog.logging",
        "cryptography",
    ],
    extras_require={
        "docs": [
            "Sphinx",
        ],
        "test": [
            "pytest-cache",
            "pytest-cov",
            "pytest-flake8",
            "pytest-rerunfailures",
            "pytest-sugar",
            "pytest-asyncio",
            "pytest",
            "coverage",
            # https://github.com/PyCQA/flake8/issues/1419#issuecomment-947243876
            "flake8<4",
        ],
    },
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="risclog.cryptography",
    name="risclog.cryptography",
    packages=find_packages("src"),
    namespace_packages=["risclog"],
    package_dir={"": "src"},
    url="https://github.com/risclog-solution/risclog.cryptography",
    version="1.1",
    zip_safe=False,
)
